module ds_intf_bit(
    clk         ,
    rst_n       ,
    rst_en      ,
    wr_en       ,
    wdata       ,
    rd_en       ,
    dq_in       ,

    rdata       ,
    rdata_vld   ,
    dq_out      ,
    dq_out_en   ,
    rdy         
    );

    //
    parameter      DATA_W =         8;
    parameter      CNT_W  =        17;

    parameter      TIME_RST       =           50_000;//1000us
    parameter      TIME_RST_LOW   =           37_500;//750us

    parameter      TIME_WR        =             3250;//62us
    parameter      TIME_WR_INSTR  =              600;//15us
    parameter      TIME_WR_DATA   =             3100;//60us

    parameter      TIME_RD        =             3250;//62us
    parameter      TIME_RD_INSTR  =               70;//1us
    parameter      TIME_RD_GET    =              700;//14us
    parameter      TIME_RD_DAT    =             3000;//60us

    //źŶ
    input                      clk    ;
    input                      rst_n  ;
    input                      rst_en ;
    input                      wr_en  ;
    input                      wdata  ;
    input                      rd_en  ;
    input                      dq_in  ;

    



    wire                      rst_en ;
    wire                      wr_en  ;
    wire                      wdata  ;
    wire                      rd_en  ;
    wire                      dq_in  ;
   
    //źŶ
    output              rdata     ;
    output              rdata_vld ;
    output              dq_out    ;
    output              dq_out_en ;
    output              rdy       ;

    //źreg
    reg             rdata     ;
    reg             rdata_vld ;
    reg             dq_out    ;
    reg             dq_out_en ;
    reg             rdy       ;

    //мźŶ
    reg   [CNT_W-1:0]           cnt;
    wire                    add_cnt;
    wire                    end_cnt;

    reg                     rdata_r;
    reg     [1:0]          flag_sel;
    reg   [CNT_W-1:0]           x  ;
    reg   [CNT_W-1:0]           y  ;
    reg                         zz ;
    reg   [CNT_W-1:0]           uu ;

    reg                  flag_work;
    wire                  rst_start;
    wire                   rd_start;
    wire                   wr_start;

    reg                     wdata_r;

    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt <= 0;
        end
        else if(add_cnt)begin
            if(end_cnt)
                cnt <= 0;
            else
                cnt <= cnt + 1;
        end
    end

    assign add_cnt = flag_work;       
    assign end_cnt = add_cnt && cnt== x-1;   

    assign rst_start  =flag_work==0&&(rst_en==1&&wr_en==0&&rd_en==0);
    assign rd_start   =flag_work==0&&(rst_en==0&&wr_en==0&&rd_en==1);
    assign wr_start   =flag_work==0&&(rst_en==0&&wr_en==1&&rd_en==0);
    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            flag_work <= 0;
        end
        else if(rst_start||rd_start||wr_start)begin



            flag_work <= 1; 
        end
        else if(end_cnt)begin
            flag_work <= 0;
        end
    end
    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            flag_sel <= 2'd0;
        end
        else if(rst_start)begin
            flag_sel <= 2'd0;
        end
        else if(wr_start)begin
            flag_sel <= 2'd1;
        end
        else if(rd_start)begin
            flag_sel <= 2'd2;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wdata_r <= 1'b0;
        end
        else if(wr_start) begin
            wdata_r <= wdata; 
        end
    end
    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            dq_out <= 1;
        end
        else if(rst_start||wr_start||rd_start)begin
            dq_out <= 0;
        end
        else if(add_cnt&&cnt==y-1)begin
            dq_out <= zz;
        end
        else if(end_cnt||(flag_sel==1&&add_cnt&&cnt==TIME_WR_DATA-1))begin
            dq_out <= 1;
        end    
    end
    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            dq_out_en <= 0;
        end
        else if(rst_start||wr_start||rd_start)begin
            dq_out_en <= 1;
        end
        else if(add_cnt&&cnt==uu-1)begin
            dq_out_en <= 1;
        end
        else if(end_cnt)begin
            dq_out_en <= 0;
        end    
    end

    //߼д
    always  @(*)begin
        if(flag_sel==2'd0)begin
            x  = TIME_RST;
            y  = TIME_RST_LOW;
            zz = 1;
            uu = TIME_RST_LOW;   
        end
        else if(flag_sel==2'd1)begin
            x  = TIME_WR;  
            y  = TIME_WR_INSTR;
            zz = wdata_r;
            uu = TIME_WR_DATA;
        end
        else begin
            x  = TIME_RD;       
            y  = TIME_RD_INSTR; 
            zz = 1;             
            uu = TIME_RD_INSTR; 
        end
    end
    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            rdata <= 0;
        end
        else if(flag_sel==2'd2&&add_cnt&&cnt==TIME_RD_GET-1)begin
            rdata <= dq_in;
        end
    end
    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            rdata_vld <= 1'b0;
        end
        else if(flag_sel==2'd2&&add_cnt&&cnt==TIME_RD_GET-1)begin
            rdata_vld <= 1'b1;
        end
        else begin
            rdata_vld <= 1'b0;
        end
    end
    
    always  @(*)begin
        if(rst_en||wr_en||rd_en||flag_work)begin
            rdy <= 1'b0;
        end
        else begin
            rdy <= 1'b1;
        end
    end






endmodule

