module seg_disp(
    clk    ,
    rst_n  ,
    disp_en,
    din    ,
    din_vld,

    seg_sel,
    segment
    );

    //
    parameter      SEG_WID          =           8;
    parameter      SEG_NUM          =           8;
    parameter      COUNT_WID        =          20;
    parameter      TIME_20UM        =        1000;

    parameter     NUM_0    =     8'b1100_0000;
    parameter     NUM_1    =     8'b1111_1001;
    parameter     NUM_2    =     8'b1010_0100;
    parameter     NUM_3    =     8'b1011_0000;
    parameter     NUM_4    =     8'b1001_1001;
    parameter     NUM_5    =     8'b1001_0010;
    parameter     NUM_6    =     8'b1000_0010;
    parameter     NUM_7    =     8'b1111_1000;
    parameter     NUM_8    =     8'b1000_0000;
    parameter     NUM_9    =     8'b1001_0000;
    parameter     NUM_10   =     8'b1000_1000;
    parameter     NUM_11   =     8'b0000_0000;
    parameter     NUM_12   =     8'b1100_0110;
    parameter     NUM_13   =     8'b0100_0000;
    parameter     NUM_14   =     8'b1000_0110;
    parameter     NUM_15   =     8'b1000_1110;
    parameter     NUM_F    =     8'b1011_1111;
    parameter     NUM_ERR  =     8'b1000_0110;


    //źŶ
    input               clk    ;
    input               rst_n  ;
    input               disp_en;
    input[31:0]         din    ;
    input[SEG_WID-1:0]  din_vld;
    
    //źwire
    wire               disp_en;
    wire[31:0]         din    ;
    wire[SEG_WID-1:0]  din_vld;   

    //źŶ
    output[SEG_WID-1:0] seg_sel;
    output[SEG_WID-1:0] segment;

    //źreg
    reg   [SEG_WID-1:0] seg_sel;
    reg   [SEG_WID-1:0] segment;

    //мźŶ
    reg   [COUNT_WID-1:0]            cnt0;
    wire                        add_cnt0 ;
    wire                        end_cnt0 ;

    reg   [SEG_WID-1:0]              cnt1;
    wire                        add_cnt1 ;
    wire                        end_cnt1 ;

    reg     [SEG_WID*4-1:0]     din_ff0  ;
    reg     [3:0]               seg_tmp  ;
    reg     [SEG_WID-1:0]              ii;

    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt0 <= 0;
        end
        else if(add_cnt0)begin
            if(end_cnt0)
                cnt0 <= 0;
            else
                cnt0 <= cnt0 + 1;
        end
    end

    assign add_cnt0 = 1;
    assign end_cnt0 = add_cnt0 && cnt0== TIME_20UM-1;

    always @(posedge clk or negedge rst_n)begin 
        if(!rst_n)begin
            cnt1 <= 0;
        end
        else if(add_cnt1)begin
            if(end_cnt1)
                cnt1 <= 0;
            else
                cnt1 <= cnt1 + 1;
        end
    end

    assign add_cnt1 = end_cnt0;
    assign end_cnt1 = add_cnt1 && cnt1== SEG_NUM-1;

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            seg_sel <= {SEG_NUM{1'b1}};
        end
        else if(disp_en)begin
            seg_sel <= ~(1'b1 << cnt1);
        end
        else begin
            seg_sel <= {SEG_NUM{1'b1}};
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            din_ff0 <= 0;
        end
        else begin
            for(ii=0;ii<SEG_NUM;ii=ii+1)begin
                if(din_vld[ii]==1'b1)begin
                    din_ff0[(ii+1)*4-1 -:4] <= din[(ii+1)*4-1 -:4];
                end
                else begin
                    din_ff0[(ii+1)*4-1 -:4] <= din_ff0[(ii+1)*4-1 -:4];
                end
            end        
        end 
    end
    
    //߼д
    always@(*)begin
        seg_tmp = din_ff0[(cnt1+1)*4-1 -:4];
    end

    //ʱ߼д
    always@(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            segment <= NUM_0;
        end
        else begin
            case(seg_tmp)
                0:   segment <=  NUM_0  ;  
                1:   segment <=  NUM_1  ;   
                2:   segment <=  NUM_2  ;   
                3:   segment <=  NUM_3  ;   
                4:   segment <=  NUM_4  ;   
                5:   segment <=  NUM_5  ;   
                6:   segment <=  NUM_6  ;   
                7:   segment <=  NUM_7  ;   
                8:   segment <=  NUM_8  ;   
                9:   segment <=  NUM_9  ;   
             4'hf:   segment <=  NUM_F  ;   
             default:segment <=  NUM_ERR;
           endcase
        end
    end

endmodule

